﻿/**
  * Photoshop連動 ActionScript
  * 
  * 保存時に出力JPG画像とサムネイルJPG画像を生成する
  * 
  * ---------------------------------
  * バージョン:1.0.4.2 2023/07/18
  * ---------------------------------
  * 
  * 
  * インストール先はEXEフォルダとPhotoshopにも入れてもらう
  * Ver 24
  * C:\Program Files\Adobe\Adobe Photoshop 2023\Presets\Scripts
  * 　　　　　　　　　　　　　　　　　　　↑↑↑Photoshopのバージョンに合わせること
  * Ver 23
  * C:\Program Files\Adobe\Adobe Photoshop 2022\Presets\Scripts
  * 　　　　　　　　　　　　　　　　　　　↑↑↑Photoshopのバージョンに合わせること
  * Ver 22
  * C:\Program Files\Adobe\Adobe Photoshop 2021\Presets\Scripts
  * 　　　　　　　　　　　　　　　　　　　↑↑↑Photoshopのバージョンに合わせること
  * Ver 21
  * C:\Program Files\Adobe\Adobe Photoshop 2020\Presets\Scripts
  * 　　　　　　　　　　　　　　　　　　　↑↑↑Photoshopのバージョンに合わせること
  * Ver 20
  * C:\Program Files\Adobe\Adobe Photoshop CC 2019\Presets\Scripts
  * 　　　　　　　　　　　　　　　　　　　↑↑↑Photoshopのバージョンに合わせること
  * 
  * Ver CS5 64bit
  * C:\Program Files\Adobe\Adobe Photoshop CS5 (64 Bit)\Presets\Scripts
  * 　　　　　　　　　　　　　　　　　　　↑↑↑Photoshopのバージョンに合わせること
 **/

//----- 定数 -----
const LOG_PATH_DEFAULT = '~/Labonetwork/ConvertPsdToJpg/';    // 保存先が決定できなかった場合に使う、/で終わること
const LOG_ROTATION_DAYS = 14;                                       // ログファイルのローテーション
const CR = String.fromCharCode(13);                                 // 改行コード
const FN_JS = "ConvertPsdToJpgJS";                                  // ログファイル名
const IC_TP = "SINGLE";                                             // ログに出力する区分（単体用かALL用か）
const LAYER_HIDEEN_LIST = ['情報'];                                 // 非表示にするレイヤーの名前リスト
const LAYER_HIDEEN_LIST_COUNT = LAYER_HIDEEN_LIST.length;
const SCRIPT_NAME ="ConvertPsdToJpgAll";

var imagePath = '';
var doq ='';
var i = 0;
var j = 0;
var k = 0;
var m = 0;
var layercount = 0;
var infoLayerCount = 0;
var documentCount = 0;
var exeHiddenList =[];

try {

    //開かれているドキュメントの個数を調べる
    documentCount = documents.length;
    
    for(i=0; i<documentCount; i++) {

        //処理するファイル指定する
        //app.documents[0]は開かれているファイルの一番左側　ファイルを閉じるとapp.documents[0]の値は変化する
        app.activeDocument = app.documents[0];
        doq = app.activeDocument;
        imagePath = String(doq.path);

        //アクティブファイルで非表示したレイヤーの変数の初期化
        exeHiddenList.splice(0);

        //logファイル作成
        logfileArray = LogFileOpen(LOG_PATH_DEFAULT);
        LogFileWrite(logfileArray,SCRIPT_NAME+"スクリプトを開始します");
        LogFileWrite(logfileArray,doq.name+"ファイルの処理開始");

        //レイヤーの個数を求める
        layerCount = doq.layers.length;

        //画像のレイヤーを非表示にする　現在は1階層のみで判定
        for(j=0; j<layerCount; j++){
            //表示だったら、非表示に変更
            if(doq.layers[j].visible == true){
            //非表示するリストと比較して含まれていたら非表示処理する
            for(k=0; k<LAYER_HIDEEN_LIST_COUNT; k++) {
                    if(doq.layers[j].name == LAYER_HIDEEN_LIST[k]){
                        LogFileWrite(logfileArray,doq.layers[j].name+"レイヤーを非表示にします");
                        //非表示にしたレイヤーを記憶する
                        exeHiddenList.push(j);
                        doq.layers[j].visible = false;
                        break;
                    }
                }
            }
        }

        //非表示にした画像を拡張子を変更して保存する
        saveFlg = saveFileJpegExtension(doq);
        if(saveFlg == false){
            throw new Error(doq.name+"ファイルの保存に失敗したため処理を停止しました");
        }

        LogFileWrite(logfileArray,doq.name+"ファイルの処理終了");
        
        //非表示にしたレイヤーを表示して保存する
        for(m=0; m<exeHiddenList.length; m++){
        doq.layers[exeHiddenList[m]].visible = true;
        }

        //保存してファイルを閉じる
        doq.close(SaveOptions.SAVECHANGES);
    }
    alert("すべてのファイルの処理が終了しました");

} catch (e) {
    alert(e.message);
    LogFileWrite(logfileArray,e.message);

} finally {
    // ログクローズ
    LogFileWrite(logfileArray,SCRIPT_NAME+"スクリプトの処理を終了します");
    LogFileClose(logfileArray);
    
}

//----- 以下、メイン処理では無い -----

/**
 * jpg形式ファイル保存
 * 
 * @param doq 開かれているファイル
 * @return true or false 保存出来たかどうか
 */
function saveFileJpegExtension(doq) {
    
    //保存するファイルの設定
    extension = "."+String(doq.fullName).split(".")[1];
    savePath = String(doq.fullName).replace(extension, '.jpg');
    fileObj = new File(savePath);
    jpegOpt = new JPEGSaveOptions();
    jpegOpt.embedColorProfile = true;
    jpegOpt.quality = 12;
    jpegOpt.formatOptions = FormatOptions.STANDARDBASELINE;
    jpegOpt.scans = 3;
    jpegOpt.matte = MatteType.NONE;

    try {
        doq.saveAs(fileObj, jpegOpt, true, Extension.LOWERCASE);
        LogFileWrite(logfileArray, "ファイルの保存に成功しました:" + decodeURI(savePath));
    
    } catch  (err) {
        LogFileWrite(logfileArray, "ファイルの保存に失敗しました:" + decodeURI(savePath));
        LogFileWrite(logfileArray, "ファイルの保存に失敗しました:" + err);
        return false;
    }
    return true;
}

/**
  * ログファイルのオープン処理
  * ret  = true / false
  * logFileName  = ログファイル名
  * fileObj  = Fileオブジェクト
  **/
function LogFileOpen(logPath) {
    var ret = false;
    var logPathName = '';

    // ダイアログから入力したフォルダpathからlog出力するpathを生成
    logPathName = logPath;
    logFileName = logPathName + FN_JS + ".log";

    //logを保存するフォルダが存在するかを確認する
    folderObj = new Folder(logPathName);
    flag = folderObj.exists;
    if (flag !== true)
    {
        folderObj.create();
    }

    // 本日を取得
    var DateObj = new Date();
    // 本日の時刻を切り捨てる
    var objToDayDate = new Date(DateObj.getFullYear(), DateObj.getMonth(), DateObj.getDate());
    logFileName = logFileName + "." + FormatDate(objToDayDate, 'yyyyMMdd');
    // 当該ファイルの日付数値を入れる箱
    var objFileDate = new Date();
    var elapsed = 0;

    // ローテーション
    folderObj = new Folder(logPathName);
    if (folderObj != null) {
        var fList = folderObj.getFiles(FN_JS + '.log.*'); //フォルダ内容取得
        var numList = fList.length; //内容の総数
        var cnt, objName, objType;
        for (cnt = 0; cnt < numList; cnt++) {
            //オブジェクトの名前
            objName = fList[cnt].name;
            //オブジェクトのタイプ
            objType = fList[cnt].constructor.name;
            // 当該ファイルの日付数値
            objFileDate = new Date(parseInt(objName.substr(-8, 4)),
                    (parseInt(objName.substr(-4, 2)) - 1),
                    parseInt(objName.substr(-2, 2)));
            // 経過日数を取得する（経過日数 ＝ 本日 － 当該ファイル名）
            elapsed = (objToDayDate - objFileDate) / 86400000;
            // 削除するのか判断する
            if (elapsed > LOG_ROTATION_DAYS) {
                fileRemoveObj = new File(logPathName + objName);
                // 削除
                fileRemoveObj.remove();

            }
        }
    }

    fileObj = new File(logFileName);
    // mode = w:write only,a:append 
    flag = fileObj.open("a");
    if (flag == true) {
        ret = true;
    } else {
        ret = false;
        alert("ログファイルが開けませんでした" + CR + logFileName);
    }

    return [ret, logFileName, fileObj];

}

/**
  * ログファイルのクローズ処理
  **/
function LogFileClose(logfileArray) {
    // ログファイルがオープン成功の場合のみクローズする
    if (logfileArray[0] == true) {
        logfileArray[2].close();
    }

}

/**
  * ログファイルの書き込み
  **/
function LogFileWrite(logfileArray, strLog) {
    var strLogLine = "";
    var dateObj = new Date();

    try {
        // 現在の時刻
        strLogLine = strLogLine + "[" + FormatDate(dateObj, 'yyyy/MM/dd HH:mm:ss:SSS') + "]";
        strLogLine = strLogLine + " - [" + IC_TP + "]" + strLog;

    }
    finally {
        // ログファイルがオープン成功の場合のみ出力する
        if (logfileArray[0] == true) {
            logfileArray[2].writeln(strLogLine);
        }
    }
}

/**
  * 日付を指定した書式の文字列にフォーマット
  * https://zukucode.com/2017/04/javascript-date-format.html
  * date: 日付オブジェクト
  * format: 書式フォーマット
  * console.log(FormatDate(date, 'yyyyMMddHHmmssSSS')); // "20170102030405006"
  * console.log(FormatDate(date, 'yyyy/MM/dd'));        // "2017/01/02"
  * console.log(FormatDate(date, 'HH:mm:ss:SSS'));      // "03:04:05:006"
  **/
function FormatDate (date, format) {
  format = format.replace(/yyyy/g, date.getFullYear());
  format = format.replace(/MM/g, ('0' + (date.getMonth() + 1)).slice(-2));
  format = format.replace(/dd/g, ('0' + date.getDate()).slice(-2));
  format = format.replace(/HH/g, ('0' + date.getHours()).slice(-2));
  format = format.replace(/mm/g, ('0' + date.getMinutes()).slice(-2));
  format = format.replace(/ss/g, ('0' + date.getSeconds()).slice(-2));
  format = format.replace(/SSS/g, ('00' + date.getMilliseconds()).slice(-3));
  return format;
};

